(in-package :sql)


(defun sql-and-qualifier (qualifier)
  (cond ((null qualifier)
         nil)
        ((= 1 (length qualifier))
         (car qualifier))
        (t
         (apply #'maisql-sys::sql-and (remove-if-not #'identity
                                                     qualifier )))))


(defun check-qualifier (qualifier)
  (if (consp qualifier)
      (case (length qualifier)
        (0 qualifier)
        (1 (first qualifier))
        (t (apply #'maisql-sys::sql-and qualifier)))
      qualifier))

(defun qualifier-functions (list)
  (mapcar (lambda (elt)
            `(when ,(car elt)
              (let ((term ,@(cdr elt)))
                (when term
                  (push term qualifier)))))
          list))

(defun qualifier-keywords (list)
  (mapcar #'car list))

(defmacro defquery ((&key function-name class-name ordering limit view-qualifier (distinct t)) &rest qualifiers)
  `(defun ,function-name (&key join all qualifier (result-class 'list)
                          (row-limit ,limit) (order-by ,ordering) order-by-descending ,@(qualifier-keywords qualifiers))
     (flet ((%select (qualifier)
                     (when join
                       (push join qualifier))
                     (when ,view-qualifier
                       (push ,view-qualifier qualifier))
                     (let* ((and-qualifier (check-qualifier qualifier))
                            (args (list ',class-name
                                        :where and-qualifier
                                        :distinct ,distinct
                                        :order-by (and (not order-by-descending) order-by)
                                        :order-by-descending order-by-descending
                                        :limit row-limit)))
                       (values (apply #'select args)
                               args))))
       (unless (or all qualifier)
         ,@(qualifier-functions qualifiers))
       (multiple-value-bind (results args)
           (%select qualifier)
         (case result-class
           (list
            results)
           (t
            (change-class (make-instance 'sql-result-set
                                         :rows results
                                         :qualifier (lambda ()
                                                      (apply #'select args))) result-class)))))))

(defclass sql-result-set ()
  ((rows
    :initarg :rows
    :initform nil)
   (row-function
    :initarg :qualifier
    :initform nil)))
