;;; -*- Syntax: Ansi-Common-Lisp; Base: 10; Mode: lisp; Package: wco -*-
;;; $Id: test-context.lisp,v 1.10 2002/12/31 17:36:48 jesse Exp $
;;;
;;; Copyright (c) 2000, 2001, 2002 onShore Development, Inc.

(in-package :sql-sys)

;; Preliminary definitions for our tests to work on.

(def-view-class foo ()
  ((foo-oid    :db-kind :key
               :initarg :oid
               :type integer)
   (test-val   :type string
               :nulls-ok t
               :initform nil
               :initarg :test-val))
  (:schemas "sql-tests"))


(defregression (:context 1)
  "create object"
  #+craig-db
  (with-test-db ()
      (let ((oid (sequence-next (sequence-from-class 'foo)))
            (context (make-instance 'editing-context :stores (list *default-database*))))
        (with-context (context)
          (make-instance 'foo :oid oid))
        (ec-reset context)
        (with-context (context)
          (unless (instance-by-key (list 'foo oid))
           (error "EC object creation failed."))))))



(defregression (:context 2)
    "modify object"
  #+craig-db
  (with-test-db ()
      (let ((oid (sequence-next (sequence-from-class 'foo)))
            (context (make-instance 'editing-context :stores (list *default-database*)))
            (foo nil))
        (with-context (context)
          (setq foo (make-instance 'foo :oid oid)))
        (with-context (context)
          (setf (slot-value foo 'test-val) "foogle"))
        (with-context (context)
          (unless (equal (slot-value (instance-by-key (list 'foo oid)) 'test-val) "foogle")
            (error "EC object modification failed."))))))

(defregression (:context 3)
    "modify object and rollback"
  #+craig-db
  (with-test-db ()
      (let ((oid (sequence-next (sequence-from-class 'foo)))
            (context (make-instance 'editing-context :stores (list *default-database*)))
            (foo nil))
        (with-context (context)
          (setq foo (make-instance 'foo :oid oid)))
        (ignore-errors
          (with-context (context)
            (setf (slot-value foo 'test-val) "foogle")
            (error "thrown on purpose")))
        (with-context (context)
          (unless (equal (slot-value (instance-by-key (list 'foo oid)) 'test-val) nil)
            (error "EC object modification rollback failed."))))))

(defregression (:context 4)
    "delete object"
  #+craig-db
   (with-test-db ()
      (let ((oid (sequence-next (sequence-from-class 'foo)))
            (context (make-instance 'editing-context :stores (list *default-database*)))
            (foo nil))
        (with-context (context)
          (setq foo (make-instance 'foo :oid oid)))
        (with-context (context)
          (ec-delete foo context))
        (with-context (context)
          (when (instance-by-key (list 'foo oid))
            (error "EC object deletion failed."))))))

(defvar *golly* "sandwich")

(defregression (:context 5)
    "delete-object and rollback"
  #+craig-db
   (with-test-db ()
      (let ((oid (sequence-next (sequence-from-class 'foo)))
            (context (make-instance 'editing-context :stores (list *default-database*)))
            (foo nil)
            (y *golly*))
        (with-context (context)
          (setq foo (make-instance 'foo :oid oid)))
        (handler-case
            (with-context (context)
              (ec-delete foo context)
              (when y (error "foo")))
          (error ()
            ))
        (with-context (context)
          (unless (instance-by-key (list 'foo oid))
            (error "EC object deletion rollback failed."))))))
