;;; -*- Syntax: Ansi-Common-Lisp; Base: 10; Mode: lisp; Package: wco -*-
;;; $Id: test-frames.lisp,v 1.1 2002/05/08 13:54:15 craig Exp $
;;;
;;; Copyright (c) 2000, 2001, 2002 onShore Development, Inc.

(in-package :sql-tests)

;; Preliminary definitions for our tests to work on.

(defregression ('frames 1)
    "create and add property"
    (with-context ((make-editing-context))
      (with-transaction ()
	(let ((fr (make-frame)))
	  (fadd fr 'foo 1)
	  (fadd fr 'foo 2)
	  (let ((res (fget fr 'foo)))
	    (unless (= (length res) 2)
	      (error "fget results should have two values: ~A" res))
	    (unless (= (car res) 1)
	      (error "fget results should be in order of addition: ~A" res)))))))

(defregression ('frames 2)
    "create and check transaction handling"
    (let ((fr nil)
	  (context (make-editing-context)))
      (with-context (context)
	(with-transaction ()
	  (setf fr (make-frame))
	  (fadd fr 'foo 1)))
      (handler-case
	  (with-context (context)
	    (with-transaction ()
	      (fadd fr 'foo 2)
	      (error "foo")))
	(error ()
	  t))
      (with-context (context)
	(unless (numberp (fget fr 'foo))
	  (break)
	  (error "transaction rollback failure.")))))

(defregression ('frames 3)
    "create and check transaction handling for fzap"
    (let ((fr nil))
      (with-transaction ()
	(setf fr (make-frame))
	(fadd fr 'foo 1))
      (handler-case
	  (with-transaction ()
	    (fadd fr 'foo 2)
	    (fzap fr 'foo)
	    (error "foo"))
	(error ()
	  t))
      (with-transaction ()
	(unless (numberp (fget fr 'foo))
	  (error "transaction rollback failure.")))))
      
(defregression ('frames 4)
    "fzap"
  (let ((fr nil))
    (with-transaction ()
      (setf fr (make-frame))
      (fadd fr 'foo 1))
    (with-transaction ()
      (fzap fr 'foo))
    (when (fget fr 'foo)
      (error "fzap failed."))))


(defregression ('frames 5)
    "ftest"
  (let ((fr nil))
    (with-transaction ()
      (setf fr (make-frame))
      (fadd fr 'foo 1)
      (unless (ftest fr 'foo 1)
	(error "ftest failed.")))))
    
(defregression ('frames 6)
    "fget test for various data types"
    (let ((fr nil))
      (with-transaction ()
	(setf fr (make-frame))
	(dolist (val (list 1 "bar" fr (list 1 2 3) :keyword 'bar nil t))
	  (fadd fr 'foo val)
	  (let ((newval (car (last (listify (fget fr 'foo))))))
	    (unless (equal val newval)
	      (error "Storage of value: ~A corrupted as ~A" val newval)))))))
