(in-package :sql-sys)

(defvar *test-db* nil)

(defvar *test-database-name* "sql-test"
  "The name of the database used by the framework.")

(defvar *test-database-user* nil
  "The username for the database used by the framework.")

(defvar *test-database-password* ""
  "The password for the database used by the framework.")

(defvar *test-database-type* :postgresql)

(defun sql-tests-connection-dictionary ()
  (let ((dbname *test-database-name*)
        (dbpassword *test-database-password*)
        (dbtype *test-database-type*)
        (username (or *test-database-user*
                      (cdr (assoc :USER ext::*environment-list*)))
                      ))
    (case dbtype
      (:postgresql
       `(nil ,dbname ,username nil))
      (:oracle
       `(,username ,dbpassword ,dbname))
      (t
       (error #i"Unrecognized database type: ~A" dbtype)))))

(defun init/test-database (&key (database-type *test-database-type*)
                           (connection-dictionary (sql-tests-connection-dictionary)))
  (ext:run-program "createdb" (list *test-database-name*))
  (setq *test-db* (sql:connect connection-dictionary
                               :database-type database-type
                               :make-default nil
                               :if-exists :old))
  (with-database (*test-db*)
    (execute-command "set datestyle to 'iso'" :database *test-db*)
    (sql-sys::ensure-schema "sql-tests"))
  *test-db*)
  
(defmacro with-test-db ((&rest backends) &body body)
  (let ((backends-defined (when backends t)))
    `(progn
      (unless *test-db*
        (init/test-database)
        (setf (sql-sys::database-schema *test-db*) "sql-tests"))
      (with-database (*test-db*)
        (if (and ,backends-defined
                 (not (member (sql-sys::database-type *test-db*) ',backends)))
            (warn "This feature is not supported on ~a databases"
                  (sql-sys::database-type *test-db*))
          ,@body)))))



